@@ -8,110 +8,17 @@ from textwrap import dedent
 import time
 
 import boto3
-from flask import Flask, request, send_from_directory
+from flask import Flask, make_response, redirect, request, send_from_directory
 from jinja2 import Template
+from sectile import Sectile
+from werkzeug.exceptions import NotFound
 
-from . import Flourish, __version__
+from . import Flourish, __version__, blueprint
 from .examples import example_files
 from .lib import relative_list_of_files_in_directory
+from .dirtrie import DirTrie
 
-
-recipe_html = Template(dedent("""\
-    <html>
-    <head>
-      <style>
-        #dimensions dt {
-          float: left;
-          margin-right: 20px;
-          min-width: 100px;
-        }
-        #dimensions dd {
-          font-weight: bold;
-        }
-        #template ol {
-          margin: 0;
-          padding: 0;
-        }
-        #template ol ol li {
-          padding: 0 0 0 17px;
-          border-left: 1px solid #999;
-          margin-left: 3px;
-          margin-bottom: 15px;
-        }
-        #template li {
-          list-style: none;
-          margin-top: 10px;
-        }
-        #template li i {
-          color: #66a;
-          font-size: 0.8em;
-        }
-        #template li.missing {
-          margin-bottom: 10px;
-        }
-        #template li.missing > i {
-          font-weight: bold;
-          color: #f99;
-        }
-        #template li pre {
-          margin: 0;
-        }
-      </style>
-    </head>
-    <body>
-      <h1>Recipe for {{path|e}}</h1>
-      <div id='template'>
-        {% if sectile_dimensions %}
-          <h2>Dimensions</h2>
-          <dl id='dimensions'>
-          {% for dim in sectile_dimensions %}
-            <dt>{{dim}}</dt>
-            <dd>{{sectile_dimensions[dim]}}</dd>
-          {% endfor %}
-          </dl>
-        {% endif %}
-        <h2>Template: <i>{{template_name|e}}</i></h2>
-        {% if sectile_fragments %}
-          {% set depth = namespace(previous=-1) %}
-          {% for fragment in sectile_fragments %}
-            {% if depth.previous < fragment.depth %}
-              <ol>
-            {% elif depth.previous > fragment.depth %}
-              {% for i in range(fragment.depth, depth.previous) %}
-                </li></ol>
-              {% endfor %}
-            {% else %}
-              </li>
-            {% endif %}
-            <li class='{% if not fragment.found %}missing{% endif %}'>
-            {% if not fragment.found %}
-              <i>{{fragment.file}}: None</i>
-            {% else %}
-              <i>{{fragment.found}}</i>
-              <pre>{{fragment.fragment|e}}</pre>
-            {% endif %}
-
-            {% if depth.previous < fragment.depth %}
-
-            {% elif depth.previous > fragment.depth %}
-            {% else %}
-              </li>
-            {% endif %}
-            {% set depth.previous = fragment.depth %}
-          {% endfor %}
-          </ol>
-          {% for i in range(depth.previous) %}
-            </li></ol>
-          {% endfor %}
-          <p>Result:</p>
-        {% endif %}
-        <pre>{{template|e}}</pre>
-      </div>
-      <h2>Context:</h2>
-      <div id='context'>{{context|e}}</div>
-    </body>
-    </html>
-    """))
+REDIRECT_ETAG='"d41d8cd98f00b204e9800998ecf8427e"'      # 0 byte file
 
 
 def main():
@@ -233,6 +140,15 @@ def main():
         action='store_true',
         help='invalidate uploaded paths in CloudFront',
     )
+    parser_upload.add_argument(
+        '--max-invalidations',
+        type=int,
+        default=100,
+        help=(
+            'maximum count of invalidation paths to request (will'
+            'collapse multiple paths to /a/b/* if count is exceeded)'
+        ),
+    )
     parser_upload.add_argument(
         'bucket',
         nargs='?',
@@ -323,32 +239,114 @@ def preview_server(args):
     def send_file(path=''):
         generate = '/%s' % path
 
-        if 'showrecipe' in request.args:
-            return recipe_html.render(flourish.path_recipe(generate))
+        try:
+            not_found_page = flourish.site_config['404_page']
+            if not_found_page.startswith('/'):
+                not_found_page = not_found_page[1:]
+        except:
+            not_found_page = None
+
+        if generate in flourish.redirects:
+            return redirect(flourish.redirects[generate])
+
+        if 'blueprint' in request.args:
+            response = make_response(
+                Template(
+                    blueprint.template
+                ).render(
+                    flourish.path_blueprint(generate)
+                )
+            )
+            response.headers.set('X-Blueprint-Page', 'true')
+            return response
         else:
             # fix URLs for .html
-            filename = os.path.basename(path)
-            if filename == '':
-                path = '%sindex.html' % path
-            _, ext = os.path.splitext(path)
+            send = path
+            if '' == os.path.basename(path):
+                send = send + 'index.html'
+            _, ext = os.path.splitext(send)
             if ext == '':
-                path += '.html'
+                send += '.html'
 
             # regenerate if requested
             if args.generate:
+                try:
+                    os.remove(os.path.join(output_dir, send))
+                except FileNotFoundError:
+                    pass
                 flourish.generate_path(generate, report=True)
 
-            if os.path.exists(os.path.join(output_dir, path)):
-                return send_from_directory(output_dir, path)
+            if os.path.exists(os.path.join(output_dir, send)):
+                return send_from_directory(output_dir, send)
             else:
-                return send_from_directory(source_dir, path)
+                if not_found_page:
+                    try:
+                        return send_from_directory(source_dir, send)
+                    except NotFound:
+                        # avoid looping on 404 fetching 404
+                        if path != not_found_page:
+                            response = send_file(not_found_page)
+                            response.status_code = 404
+                            return response
+                        else:
+                            raise NotFound
+                else:
+                    return send_from_directory(source_dir, send)
+
+    @app.route('/_sectile/update', methods=['GET', 'POST'])
+    def edit_sectile_fragment():
+        sectile = Sectile(fragments=args.fragments)
+        if request.method == 'GET':
+            return make_response(
+                    Template(
+                        blueprint.update_template
+                    ).render({
+                        'fragment': request.args['fragment'],
+                        'content': sectile.get_fragment_file(request.args['fragment']),
+                        '_return': request.args['_return'],
+                    })
+                )
+        else:
+            sectile.update_fragment(
+                request.form['fragment'],
+                request.form['content'],
+            )
+            return redirect(request.form['_return'])
+
+    @app.route('/_sectile/delete', methods=['POST'])
+    def delete_sectile_fragment():
+        sectile = Sectile(fragments=args.fragments)
+        sectile.delete_fragment(request.form['fragment'])
+        return redirect(request.form['_return'])
+
+    @app.route('/_sectile/create', methods=['POST'])
+    def create_sectile_fragment():
+        sectile = Sectile(fragments=args.fragments)
+        dimensions = {}
+        for dimension in sectile.get_dimensions_list():
+            dimensions[dimension] = request.form[dimension]
+        sectile.create_fragment(
+            request.form['fragment'],
+            request.form['path'],
+            **dimensions,
+        )
+        return redirect(request.form['_return'])
 
     @app.after_request
     def add_header(response):
+        if (
+            response.content_type.startswith('text/html')
+            and args.generate
+            and not 'X-Blueprint-Page' in response.headers
+        ):
+            response.direct_passthrough = False
+            response.data = (
+                response.data + bytes(blueprint.toolbar, 'utf-8')
+            )
         response.cache_control.max_age = 0
         return response
 
-    app.run(port=args.port)
+    app.run(port=args.port, debug=True)
 
 
 def create_example(args):
@@ -380,7 +378,6 @@ def upload(args):
             templates_dir=args.templates,
             fragments_dir=args.fragments,
             output_dir=args.output,
-            skip_scan=True,
         )
     except Flourish.MissingKey as e:
         sys.exit('Error: %s' % str(e))
@@ -403,6 +400,23 @@ def upload(args):
     for _object in _bucket.objects.all():
         _objects.update({_object.key: _object})
 
+    redirects = flourish.redirects
+    for redirect in redirects:
+        _s3path = redirect[1:]
+        if (
+            _s3path not in _objects
+            or _objects[_s3path].e_tag != REDIRECT_ETAG
+        ):
+            print('->', redirect, 'to', redirects[redirect])
+            if not args.dry_run:
+                _bucket.put_object(
+                    Key=_s3path,
+                    Body='',
+                    ContentType='text/html',
+                    WebsiteRedirectLocation=redirects[redirect],
+                )
+                _invalidations.append(redirect)
+
     for _path in _files:
         _file = os.path.basename(_path)
         _type, _ = mimetypes.guess_type(_file)
@@ -438,13 +452,17 @@ def upload(args):
                 _invalidations.append(invalidate)
 
     if args.invalidate and _invalidations:
+        trie = DirTrie()
+        for path in _invalidations:
+            trie.insert(path)
+        invalidation_paths = trie.collapse(args.max_invalidations)
         cf = boto3.client('cloudfront')
         result = cf.create_invalidation(
             DistributionId=_cloudfront,
             InvalidationBatch={
                 'Paths': {
-                    'Quantity': len(_invalidations),
-                    'Items': _invalidations,
+                    'Quantity': len(invalidation_paths),
+                    'Items': invalidation_paths,
                 },
                 'CallerReference': datetime.now().isoformat(),
             }
